/*
 * file SimpleQuery.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM - you are allowed to copy, modify and 
 * redistribute this file as part of any program that interfaces with 
 * IBM Rational CM API.
 *
 * com.ibm.rational.stp.client.samples.SimpleQuery
 *
 * (C) Copyright IBM Corporation 2004, 2008.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.stp.client.samples;

import static javax.swing.JOptionPane.showInputDialog;

import java.util.List;

import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.wvcm.ProviderFactory;
import javax.wvcm.WvcmException;

import com.ibm.rational.wvcm.stp.StpProvider;
import com.ibm.rational.wvcm.stp.StpProvider.Domain;
import com.ibm.rational.wvcm.stp.cc.CcProvider;
import com.ibm.rational.wvcm.stp.cq.CqProvider;
import com.ibm.rational.wvcm.stp.cq.CqQuery;
import com.ibm.rational.wvcm.stp.cq.CqResultSet;

/**
 * Sample CM API Application that executes a query named by the user and
 * displays the results in a passive window.
 */
public class SimpleQuery
{

    /**
     * @param args
     */
    public static void main(String[] args) throws Exception
    {
        CqProvider provider = (CqProvider) 
            ProviderFactory.createProvider(CqProvider.CQ_ONLY_PROVIDER_CLASS, g_callback);
        CqQuery query = provider.buildProxy(CqQuery.class, 
                                            showInputDialog("Enter Name of Query to Run (use indicated format)", 
                                                            "<query-pathname>@<db-set>/<user-db>"));
        CqResultSet results = query.doExecute(1, Integer.MAX_VALUE, null);
        JTextArea text = new JTextArea(640, 480);
        JFrame frame = new JFrame(query.location().toString());
    
        while (results.hasNext()) {
            for (String r: results.next().getStrings()) text.append(r + "\t");
            text.append("\n");
        }
        
        provider.terminate();
        frame.setContentPane(new JScrollPane(text));
        frame.setBounds(500, 400, 600, 300);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setVisible(true);
    }

    static StpProvider.StpCallback g_callback = new StpProvider.StpCallback() {
        public Authentication getAuthenticationEx(Domain domain,
                                                  String realm,
                                                  int retryCount,
                                                  StpProvider provider,
                                                  WvcmException failure) 
        throws WvcmException
        {
            // Try to reuse last credentials on each new repository
            if (m_unpw != null && retryCount == 0)
                return m_unpw;

            String title = "Enter " + domain + " Username '+' Password for " 
                            + realm + " [" + retryCount + "]";
            
            if (failure != null)
                title = "Login failed: " + failure + "\n" + title;
            
            String unpw = JOptionPane.showInputDialog(title, "admin+");

            if (unpw == null || unpw.length() == 0)
                throw new IllegalAccessError("User canceled request");
            
            return m_unpw = unpw.equals("anonymous")? null: new UnPw(unpw);
        }

        public Authentication getAuthentication(String r, int c)
            {   return null; /* Will not be called */   }

        private UnPw m_unpw;
    };

    /**
     * A simple Authentication object in which the username and password
     * obtained from the user is cached for use by the CM API.
     */
    static class UnPw implements CcProvider.CcAuthentication {
        /**
         * Constructs an Authentication object
         * 
         * @param unpw A String containing username+password.
         */
        UnPw(String unpw) {  m_data = unpw.split("\\+", -2); }

        public String loginName() { return m_data[0]; }
        public String password() { return m_data.length > 1 ? m_data[1] : ""; };
        public String getPrimaryGroupName() {  return m_data.length > 2 ? m_data[2] : ""; }

        /* (non-Javadoc)
         * @see com.ibm.rational.wvcm.stp.cc.CcProvider.CcAuthentication#getGroupList()
         */
        public List<String> getGroupList()
        {
            return null;
        }

        /** The cached credentials */
        private String[] m_data;

    }
}
